@@ -1,12 +1,13 @@
 from .common import *
 from .contentAPI import ContentAPI
 from .entityAPI import EntityAPI
-from .uploadAPI import UploadAPI, simple_asset_package, complex_asset_package
+from .uploadAPI import UploadAPI, simple_asset_package, complex_asset_package, cvs_to_xsd, cvs_to_xml, \
+                    cvs_to_cmis_xslt, csv_to_search_xml
 from .workflowAPI import WorkflowAPI
 
 __author__ = "James Carr (drjamescarr@gmail.com)"
 
 # Version of the Preservica API package
-__version__ = "0.8.9"
+__version__ = "0.8.11"
 
 __license__ = "Apache License Version 2.0"
